@@ -366,7 +366,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                 try:
                     start_chunkid = self.global2chunkid(start)
                 except IndexError:
-                    if start > 0:
+                    if start >= 0:
                         # case A or B start was set beyond len(self), clamp it
                         start, start_chunkid = len(self), len(self._chunks)
                     if step < 0:
@@ -430,15 +430,15 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             if len(chunks) == 0 and len(self._chunks) > 0:
                 chunks.append(self._chunks[0][(slice(0, 0),) + tail])   # so that sliced.type == self.type
 
-            return self.__class__(chunks)
+            return self.copy(chunks=chunks)
 
         else:
             head = awkward.util.numpy.array(head, copy=False)
             if len(head.shape) == 1 and issubclass(head.dtype.type, awkward.util.numpy.integer):
                 if len(head) == 0 and len(self._chunks) == 0:
-                    return self.__class__([])[tail]
+                    return self.copy(chunks=[])[tail]
                 elif len(head) == 0:
-                    return self.__class__([self._chunks[0][(slice(0, 0),) + tail]])
+                    return self.copy(chunks=[self._chunks[0][(slice(0, 0),) + tail]])
 
                 chunkid, head = self.global2chunkid(head, return_normalized=True)
 
@@ -453,7 +453,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                     for i, cid in enumerate(chunkid[mask]):
                         localindex = head[offsets[i]:offsets[i + 1]] - self.offsets[cid]
                         chunks.append(self._chunks[cid][localindex])
-                    return self.__class__(chunks)
+                    return self.copy(chunks=chunks)
 
                 elif awkward.util.isnumpy(self.type):
                     out = awkward.util.numpy.empty((len(head),) + self.type.shape[1:], dtype=self.type.dtype)
@@ -487,7 +487,7 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                         if len(x) > 0:
                             chunks.append(x)
 
-                return self.__class__(chunks)
+                return self.copy(chunks=chunks)
 
             else:
                 raise TypeError("cannot interpret shape {0}, dtype {1} as a fancy index or mask".format(head.shape, head.dtype))
@@ -586,33 +586,74 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
                     out[i] = awkward.array.objects.Methods.maybemixin(types[i], ChunkedArray)(chunks[i])
             return tuple(out)
 
-    def any(self):
-        return any(x.any() for x in self._chunks)
+    def _hasjagged(self):
+        for chunkid in range(len(self._chunks)):
+            self.knowcounts(until=chunkid)
+            if self._counts[chunkid] > 0:
+                return awkward.util._hasjagged(self._chunks[chunkid])
+        else:
+            return False
 
-    def all(self):
-        return all(x.all() for x in self._chunks)
+    def _reduce(self, ufunc, identity, dtype, regularaxis):
+        self.knowcounts()
+        self._valid()
 
-    @classmethod
-    def concat(cls, first, *rest):
-        raise NotImplementedError
+        if awkward.util._hasjagged(self):
+            chunks = []
+            for chunkid, chunk in enumerate(self._chunks):
+                this = chunk._reduce(ufunc, identity, dtype, regularaxis)
+                if len(this) > 0:
+                    chunks.append(this)
+            return self.copy(chunks=chunks)
 
-    @property
-    def base(self):
-        raise TypeError("ChunkedArray has no base")
+        out = None
+        for chunkid, chunk in enumerate(self._chunks):
+            if self._counts[chunkid] > 0:
+                this = awkward.util._reduce(chunk[:self._counts[chunkid]], ufunc, identity, dtype, regularaxis)
+                if out is None:
+                    out = this
+                else:
+                    out = ufunc(out, this)
+
+        if out is None:
+            if dtype is None:
+                return identity
+            else:
+                return dtype.type(identity)
+        else:
+            return out
+
+    def _prepare(self, identity, dtype):
+        self.knowcounts()
+        out = None
+        pos = 0
+        for chunkid, chunk in enumerate(self._chunks):
+            if self._counts[chunkid] > 0:
+                this = chunk[:self._counts[chunkid]]
+                if out is None:
+                    if dtype is None and issubclass(this.dtype.type, (awkward.util.numpy.bool_, awkward.util.numpy.bool)):
+                        dtype = awkward.util.numpy.dtype(type(identity))
+                    if dtype is None:
+                        dtype = this.dtype
+                    out = awkward.util.numpy.empty((sum(self._counts),) + this.shape[1:], dtype=dtype)
+
+                newpos = pos + this.shape[0]
+                out[pos:newpos] = this
+                pos = newpos
+
+        if out is None:
+            if dtype is None:
+                dtype = self.DEFAULTTYPE
+            return awkward.util.numpy.array([identity], dtype=dtype)
+        else:
+            return out
 
     @property
     def columns(self):
-        if len(self._chunks) == 0 or isinstance(self.type.to, awkward.util.numpy.dtype):
-            return []
-        else:
-            return self._chunks[0].columns
-
-    @property
-    def allcolumns(self):
-        if len(self._chunks) == 0 or isinstance(self.type.to, awkward.util.numpy.dtype):
-            return []
-        else:
-            return self._chunks[0].allcolumns
+        for chunkid in range(len(self._chunks)):
+            self.knowcounts(until=chunkid)
+            if self._counts[chunkid] > 0:
+                return self._chunks[chunkid].columns
 
     def astype(self, dtype):
         chunks = []
@@ -624,9 +665,6 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
             counts.append(self._counts[i])
         return self.copy(chunks=chunks, counts=counts)
 
-    def pandas(self):
-        raise NotImplementedError
-
 class AppendableArray(ChunkedArray):
     """
     AppendableArray
@@ -773,6 +811,9 @@ class AppendableArray(ChunkedArray):
             self._counts[-1] += howmany
             values = values[howmany:]
 
+    def _hasjagged(self):
+        return False
+
     def astype(self, dtype):
         chunks = []
         for chunk in self._chunks:
